import sys
import re
from dynaconf import Dynaconf
from .i18n import T

def is_valid_openai_api_key(api_key):
    """
    校验是否为有效的 OpenAI API Key 格式。
    OpenAI API Key 格式通常以 "sk-" 开头，后面是 48 个字母或数字，总长度为 51
    :param api_key: 待校验的 API Key 字符串
    :return: 如果格式有效返回 True，否则返回 False
    """
    pattern = r"^sk-[A-Za-z0-9]{48}$"
    return bool(re.match(pattern, api_key))


class ConfigManager:
    def __init__(self, default_config="default.toml", user_config="aipython.toml"):
        self.default_config = default_config
        self.user_config = user_config
        self.config = self._load_config()

    def _load_config(self):
        try:
            config = Dynaconf(
                settings_files=[self.default_config, self.user_config],
                envvar_prefix="AIPY", merge_enabled=True
            )
        except Exception as e:
            print(T('error_loading_config').format(e))
            config = None
        return config

    def get_config(self):
        return self.config

    def check_config(self):
        if not self.config:
            print(T('config_file_error'))
            return

        self.check_llm()

    def check_llm(self):
        if not self.config:
            print(T('config_not_loaded'))
            return

        llm = self.config.get("llm")
        if not llm:
            print(T('llm_config_not_found'))

        llms = {}
        for name, config in self.config.get('llm', {}).items():
            if config.get("enable", True):
                llms[name] = config

        if not llms:
            self._init_llm()

    def _init_llm(self):
        print(T('trustoken_register_instruction').format(self.user_config))

        while True:
            user_token = input(T('prompt_token_input')).strip()
            if user_token.lower() == "exit":
                print(T('exit_token_prompt'))
                sys.exit(0)
            if not user_token:
                print(T('no_token_detected'))
                continue
            if not is_valid_openai_api_key(user_token):
                print(T('invalid_token'))
                continue

            self.save_trustoken(user_token)

            self.config = self._load_config()
            break

    def save_trustoken(self, token):
        config_file = self.user_config
        try:
            with open(config_file, "a") as f:
                f.write("\n[llm.trustoken]\n")
                f.write(f'api_key = "{token}"\n')
                f.write('base_url = "https://api.trustoken.ai/v1"\n')
                f.write('model = "auto"\n')
                f.write("default = true\n")
            print(T('token_saved').format(config_file))
        except Exception as e:
            print(T('token_save_error').format(e))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import subprocess
from functools import wraps

from .i18n import T

def restore_output(func):
    @wraps(func)
    def wrapper(self, *args, **kwargs):
        old_stdout, old_stderr = sys.stdout, sys.stderr
        sys.stdout, sys.stderr = sys.__stdout__, sys.__stderr__

        try:
            return func(self, *args, **kwargs)
        finally:
            sys.stdout, sys.stderr = old_stdout, old_stderr
    return wrapper

def confirm(console, prompt, default="n", auto=None):
    if auto in (True, False):
        console.print(f"✅ {T('auto_confirm')}")
        return auto
    while True:
        response = console.input(prompt).strip().lower()
        if not response:
            response = default
        if response in ["y", "n"]:
            break
    return response == "y"

def install_packages(console, packages):
    console.print(f"[green]Install packages: {', '.join(packages)} with pip")
    cp = subprocess.run([sys.executable, "-m", "pip", "install"] + packages)
    return cp.returncode == 0


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import time
from collections import Counter, defaultdict
from dataclasses import dataclass, field
from abc import ABC, abstractmethod

import openai
import requests
import anthropic
from rich.live import Live
from rich.panel import Panel
from rich.text import Text
from rich.markdown import Markdown

from .i18n import T

@dataclass
class ChatMessage:
    role: str
    content: str
    reason: str = None
    usage: Counter = field(default_factory=Counter)

class ChatHistory:
    def __init__(self):
        self.messages = []
        self._total_tokens = Counter()

    def __len__(self):
        return len(self.messages)
    
    def json(self):
        return [msg.__dict__ for msg in self.messages]
    
    def add(self, role, content):
        self.add_message(ChatMessage(role=role, content=content))

    def add_message(self, message: ChatMessage):
        self.messages.append(message)
        self._total_tokens += message.usage

    def get_usage(self):
        return iter(row.usage for row in self.messages if row.role == "assistant")
    
    def get_summary(self):
        summary = {'time': 0, 'input_tokens': 0, 'output_tokens': 0, 'total_tokens': 0}
        summary.update(dict(self._total_tokens))
        summary['rounds'] = sum(1 for row in self.messages if row.role == "assistant")
        return summary

    def get_messages(self):
        return [{"role": msg.role, "content": msg.content} for msg in self.messages]

class BaseClient(ABC):
    MODEL = None
    BASE_URL = None

    def __init__(self, config):
        self.name = None
        self.console = None
        self.max_tokens = config.get("max_tokens")
        self._model = config.get("model") or self.MODEL
        self._timeout = config.get("timeout")
        self._api_key = config.get("api_key")
        self._base_url = config.get("base_url") or self.BASE_URL
        self._stream = config.get("stream", True)

    def __repr__(self):
        return f"{self.__class__.__name__}<{self.name}>({self._model}, {self.max_tokens})"
    
    @abstractmethod
    def get_completion(self, messages):
        pass
        
    def add_system_prompt(self, history, system_prompt):
        history.add("system", system_prompt)

    @abstractmethod
    def _parse_usage(self, response):
        pass

    @abstractmethod
    def _parse_stream_response(self, response):
        pass

    @abstractmethod
    def _parse_response(self, response):
        pass

    def parse_response(self, response):
        if self._stream:
            response = self._parse_stream_response(response)
        else:
            response = self._parse_response(response)
        return response
    
    def __call__(self, history, prompt, system_prompt=None):
        # We shall only send system prompt once
        if not history and system_prompt:
            self.add_system_prompt(history, system_prompt)
        history.add("user", prompt)

        start = time.time()
        self.console.record = False
        with self.console.status(f"[dim white]{T('sending_task', self.name)} ..."):
            response = self.get_completion(history.get_messages())
        self.console.record = True
        end = time.time()
        if response:
            msg = self.parse_response(response)
            msg.usage['time'] = round(end - start, 3)
            history.add_message(msg)
            response = msg.content
        return response

# https://platform.openai.com/docs/api-reference/chat/create
# https://api-docs.deepseek.com/api/create-chat-completion
class OpenAIClient(BaseClient):
    def __init__(self, config):
        super().__init__(config)
        self._client = openai.Client(api_key=self._api_key, base_url=self._base_url, timeout=self._timeout)

    def add_system_prompt(self, history, system_prompt):
        history.add("system", system_prompt)

    def _parse_usage(self, usage):
        usage = Counter({'total_tokens': usage.total_tokens,
                'input_tokens': usage.prompt_tokens,
                'output_tokens': usage.completion_tokens})
        return usage
    
    def _parse_stream_response(self, response):
        full_response = ""
        
        title = f"{self.name} {T('llm_response')}"
        usage = Counter()
        with Live(auto_refresh=True, vertical_overflow='visible') as live:
            status = self.console.status(f"[dim white]{self.name} {T('thinking')}...", spinner='runner')
            response_panel = Panel(status, title=title, border_style="blue")
            live.update(response_panel)
            
            for chunk in response:
                if hasattr(chunk, 'usage') and chunk.usage is not None:
                    usage = self._parse_usage(chunk.usage)

                if chunk.choices and chunk.choices[0].delta.content:
                    content = chunk.choices[0].delta.content
                    full_response += content
                    
                    try:
                        md = Markdown(full_response)
                        response_panel = Panel(md, title=title, border_style="green")
                    except Exception:
                        text = Text(full_response)
                        response_panel = Panel(text, title=title, border_style="yellow")
                    
                    live.update(response_panel)
                    
        segments = self.console.render(response_panel)
        self.console._record_buffer.extend(segments)
        return ChatMessage(role="assistant", content=full_response, usage=usage)

    def _parse_response(self, response):
        message = response.choices[0].message
        reason = getattr(message, "reasoning_content", None)
        return ChatMessage(
            role=message.role,
            content=message.content,
            reason=reason,
            usage=self._parse_usage(response.usage)
        )

    def get_completion(self, messages):
        kws = {'stream_options': {'include_usage': True}} if self._stream and not self._base_url else {}
        try:
            response = self._client.chat.completions.create(
                model = self._model,
                messages = messages,
                stream=self._stream,
                max_tokens = self.max_tokens,
                **kws
            )
        except Exception as e:
            self.console.print(f"❌ [bold red]{self.name} API {T('call_failed')}: [yellow]{str(e)}")
            response = None
        return response

# https://github.com/ollama/ollama/blob/main/docs/api.md
class OllamaClient(BaseClient):
    def __init__(self, config):
        super().__init__(config)
        self._session = requests.Session()

    def _parse_usage(self, response):
        ret = {'input_tokens': response['prompt_eval_count'], 'output_tokens': response['eval_count']}
        ret['total_tokens'] = ret['input_tokens'] + ret['output_tokens']
        return ret

    def _parse_stream_response(self, response):
        full_response = ""
        
        title = f"{self.name} {T('llm_response')}"
        with Live(auto_refresh=True, vertical_overflow='visible') as live:
            status = self.console.status(f"[dim white]{self.name} {T('thinking')}...", spinner='runner')
            response_panel = Panel(status, title=title, border_style="blue")
            live.update(response_panel)
            
            for chunk in response:
                msg = chunk.json()
                if msg['done']:
                    usage = self._parse_usage(chunk.usage)
                    break

                if 'message' in msg and 'content' in msg['message'] and msg['message']['content']:
                    content = msg['message']['content']
                    full_response += content
                    
                    try:
                        md = Markdown(full_response)
                        response_panel = Panel(md, title=title, border_style="green")
                    except Exception:
                        text = Text(full_response)
                        response_panel = Panel(text, title=title, border_style="yellow")
                    
                    live.update(response_panel)
                    
        segments = self.console.render(response_panel)
        self.console._record_buffer.extend(segments)
        return ChatMessage(role="assistant", content=full_response, usage=usage)

    def _parse_response(self, response):
        response = response.json()
        msg = response["message"]
        return ChatMessage(role=msg['role'], content=msg['content'], usage=self._parse_usage(response))
    
    def get_completion(self, messages):
        try:
            response = self._session.post(
                f"{self._base_url}/api/chat",
                json={
                    "model": self._model,
                    "messages": messages,
                    "stream": self._stream,
                    "options": {"num_predict": self.max_tokens}
                },
                timeout=self._timeout
            )
            response.raise_for_status()
        except Exception as e:
            self.console.print(f"❌ [bold red]{self.name} API {T('call_failed')}: [yellow]{str(e)}")
            response = None
        return response

# https://docs.anthropic.com/en/api/messages
class ClaudeClient(BaseClient):
    def __init__(self, config):
        super().__init__(config)
        self._client = anthropic.Anthropic(api_key=self._api_key, timeout=self._timeout)
        self._system_prompt = None

    def _parse_usage(self, response):
        usage = response.usage
        ret = {'input_tokens': usage.input_tokens, 'output_tokens': usage.output_tokens}
        ret['total_tokens'] = ret['input_tokens'] + ret['output_tokens']
        return ret

    def _parse_stream_response(self, response):
        usage = Counter()    
        full_response = ""
        
        title = f"{self.name} {T('llm_response')}"
        with Live(auto_refresh=True, vertical_overflow='visible') as live:
            status = self.console.status(f"[dim white]{self.name} {T('thinking')}...", spinner='runner')
            response_panel = Panel(status, title=title, border_style="blue")
            live.update(response_panel)
            
            for event in response:
                if hasattr(event, 'delta') and hasattr(event.delta, 'text') and event.delta.text:
                    content = event.delta.text
                    full_response += content
                    
                    try:
                        md = Markdown(full_response)
                        response_panel = Panel(md, title=title, border_style="green")
                    except Exception:
                        text = Text(full_response)
                        response_panel = Panel(text, title=title, border_style="yellow")
                    live.update(response_panel)
                elif hasattr(event, 'message') and hasattr(event.message, 'usage') and event.message.usage:
                    usage['input_tokens'] += getattr(event.message.usage, 'input_tokens', 0)
                    usage['output_tokens'] += getattr(event.message.usage, 'output_tokens', 0)
                elif hasattr(event, 'usage') and event.usage:
                    usage['input_tokens'] += getattr(event.usage, 'input_tokens', 0)
                    usage['output_tokens'] += getattr(event.usage, 'output_tokens', 0)

        usage['total_tokens'] = usage['input_tokens'] + usage['output_tokens']          
        segments = self.console.render(response_panel)
        self.console._record_buffer.extend(segments)
        return ChatMessage(role="assistant", content=full_response, usage=usage)

    def _parse_response(self, response):
        content = response.content[0].text
        role = response.role
        return ChatMessage(role=role, content=content, usage=self._parse_usage(response))
    
    def add_system_prompt(self, history, system_prompt):
        self._system_prompt = system_prompt

    def get_completion(self, messages):
        try:
            message = self._client.messages.create(
                model = self._model,
                messages = messages,
                stream=self._stream,
                system=self._system_prompt,
                max_tokens = self.max_tokens
            )
        except Exception as e:
            self.console.print(f"❌ [bold red]{self.name} API {T('call_failed')}: [yellow]{str(e)}")
            message = None
        return message

class GeminiClient(OpenAIClient): 
    BASE_URL = 'https://generativelanguage.googleapis.com/v1beta/'
    MODEL = 'gemini-2.5-pro-exp-03-25'

class DeepSeekClient(OpenAIClient): 
    BASE_URL = 'https://api.deepseek.com'
    MODEL = 'deepseek-chat'

class GrokClient(OpenAIClient): 
    BASE_URL = 'https://api.x.ai/v1/'
    MODEL = 'grok-2-latest'

class TrustClient(OpenAIClient): 
    BASE_URL = 'https://api.trustoken.ai/v1'
    MODEL = 'auto'

class LLM(object):
    CLIENTS = {
        "openai": OpenAIClient,
        "ollama": OllamaClient,
        "claude": ClaudeClient,
        "gemini": GeminiClient,
        "deepseek": DeepSeekClient,
        'grok': GrokClient,
        'trust': TrustClient
    }

    def __init__(self, console, configs, max_tokens=None):
        self.llms = {}
        self.console = console
        self.default = None
        self._last = None
        self.history = ChatHistory()
        self.max_tokens = max_tokens
        names = defaultdict(set)
        for name, config in configs.items():
            if not config.get('enable', True):
                names['disabled'].add(name)
                continue

            try:
                client = self.get_client(config)
            except Exception as e:
                self.console.print_exception()
                names['error'].add(name)
                continue
            
            names['available'].add(name)
            client.name = name
            client.console = console
            if not client.max_tokens:
                client.max_tokens = self.max_tokens
            self.llms[name] = client

            if config.get('default', False) and not self.default:
                self.default = client
                names['default'] = name

        if not self.default:
            name = list(self.llms.keys())[0]
            self.default = self.llms[name]
            names['default'] = name
        self.current = self.default
        self.names = names

    def __len__(self):
        return len(self.llms)
    
    def __repr__(self):
        return f"Current: {'default' if self.current == self.default else self.current}, Default: {self.default}"
    
    @property
    def last(self):
        return self._last.name if self._last else None
    
    def get_last_message(self, role='assistant'):
        return self.history.get_last_message(role)
    
    def clear(self):
        self.history = ChatHistory()

    def get_client(self, config):
        proto = config.get("type", "openai")
        model = config.get("model")
        max_tokens = config.get("max_tokens") or self.max_tokens
        timeout = config.get("timeout")

        client = self.CLIENTS.get(proto.lower())
        if not client:
            raise ValueError(f"Unsupported LLM provider: {proto}")
        return client(config)
    
    def __contains__(self, name):
        return name in self.llms
    
    def use(self, name):
        llm = self.llms.get(name)
        if not llm:
            self.console.print(f"[red]LLM: {name} not found")
        else:
            self.current = llm
            self.console.print(f"[green]LLM: use {name}")

    def __call__(self, instruction, system_prompt=None, name=None):
        """ LLM 选择规则
        1. 如果 name 为 None, 使用 current
        2. 如果 name 存在，使用 name 对应的
        3. 使用 default
        """
        if not name:
            llm = self.current
        else:
            llm = self.llms.get(name, self.default)
        self._last = llm
        return llm(self.history, instruction, system_prompt=system_prompt)
        


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import locale

MESSAGES = {
    'zh': {
        'start_instruction': '开始处理指令',
        'start_execute': '开始执行代码块',
        'execute_result': '执行结果',
        'start_feedback': '开始反馈结果',
        'end_instruction': '结束处理指令',
        'no_context': '未找到上下文信息',
        'upload_success': '文章上传成功',
        'upload_failed': '上传失败 (状态码: {})',
        'reset_warning': '严重警告：这将重新初始化❗❗❗',
        'reset_confirm': '如果你确定要继续，请输入 y',
        'llm_response': '响应',
        'env_description': '环境变量名称和意义',
        'description': '描述',
        'unknown_format': '不支持的文件格式',
        'ask_for_packages': '申请安装第三方包',
        'agree_packages': '如果同意，请输入',
        'ask_for_env': '申请获取环境变量 {}，用途',
        'env_exist': '环境变量 {} 存在，返回给代码使用',
        'input_env': '未找到环境变量 {}，请输入',
        'call_failed': '调用失败',
        'think': '思考',
        'publish_disabled': "当前环境不支持发布",
        'auto_confirm': '自动确认',
        'packages_exist': '申请的第三方包已经安装',
        'thinking': '正在绞尽脑汁思考中，请稍等6-60秒',
        'no_available_llm': '没有可用的 LLM，请检查配置文件',
        'banner1_python': "请用 ai('任务') 输入需要 AI 处理的任务 (输入 ai.use(llm) 切换 下述 LLM：",
        'banner1': "请输入需要 AI 处理的任务 (输入 /use 切换 下述LLM)",
        'default': '默认',
        'available': '可用',
        'ai_mode_enter': '进入 AI 模式，开始处理任务，输入 Ctrl+d 或 /done 结束任务',
        'ai_mode_exit': "[退出 AI 模式]",
        'ai_mode_unknown_command': "[AI 模式] 未知命令",
        'Task Summary': '任务总结',
        'Round': '轮次',
        'Time(s)': '时间(秒)',
        'In Tokens': '输入令牌数',
        'Out Tokens': '输出令牌数',
        'Total Tokens': '总令牌数',
        'sending_task': '正在向 {} 下达任务',
        'error_loading_config': "加载配置时出错: {}",
        'config_file_error': "请检查配置文件路径和格式。",
        'config_not_loaded': "配置尚未加载。",
        'llm_config_not_found': "缺少 'llm' 配置。",
        'trustoken_register_instruction': (
            "当前环境缺少配置文件，请注册一个trustoken账号，可以使用免费赠送的API账号。\n"
            "浏览器打开 https://api.trustoken.ai/register ， 进行账号注册。\n"
            "注册后进行登录，访问页面顶部的“令牌”页面，或者点击这个地址：https://api.trustoken.ai/token \n"
            "点击“复制”按钮，复制令牌到剪贴板。在下面进行粘贴。\n"
            "另外，也可以选择退出，然后手动编辑配置文件 {}，配置自己已有的其他大模型令牌"
        ),
        'prompt_token_input': "请粘贴令牌并按 Enter 键 (输入 'exit' 退出): ",
        'exit_token_prompt': "退出令牌输入流程。",
        'no_token_detected': "未检测到令牌输入。",
        'invalid_token': "输入的令牌不合法，请确保令牌正确，格式为‘sk-xxxxxx……’，或输入 'exit' 退出。",
        'token_saved': "令牌已保存到 {}",
        'token_save_error': "保存令牌时出错: {}"
    },
    'en': {
        'start_instruction': 'Start processing instruction',
        'start_execute': 'Start executing code block',
        'execute_result': 'Execution result',
        'start_feedback': 'Start sending feedback',
        'end_instruction': 'End processing instruction',
        'no_context': 'No context information found',
        'upload_success': 'Article uploaded successfully',
        'upload_failed': 'Upload failed (status code: {})',
        'reset_warning': 'Severe warning: This will reinitialize❗❗❗',
        'reset_confirm': 'If you are sure to continue, enter y',
        'llm_response': 'reply',
        'env_description': 'Environment variable name and meaning',
        'description': 'Description',
        'unknown_format': 'Unsupported file format',
        'ask_for_packages': 'Request to install third-party packages',
        'agree_packages': 'If you agree, please enter',
        'ask_for_env': 'Request to obtain environment variable {}, purpose',
        'env_exist': 'Environment variable {} exists, returned for code use',
        'input_env': 'Environment variable {} not found, please enter',
        'call_failed': 'Call failed',
        'think': 'Think',
        'publish_disabled': "Current environment does not support publishing",
        'auto_confirm': 'Auto confirm',
        'packages_exist': 'Third-party packages have been installed',
        'thinking': 'is thinking hard, please wait 6-60 seconds',
        'no_available_llm': 'No available LLM, please check the configuration file',
        'banner1_python': "Please use ai('task') to enter the task to be processed by AI (enter ai.use(llm) to switch to the following LLM:",
        'banner1': "Please enter the task to be processed by AI (enter /use to switch to the following LLM)",
        'default': 'Default',
        'available': 'Available',
        'ai_mode_enter': 'Enter AI mode, start processing tasks, enter Ctrl+d or /done to end the task',
        'ai_mode_exit': "[Exit AI mode]",
        'ai_mode_unknown_command': "[AI mode] Unknown command",
        'Task Summary': 'Task Summary',
        'Round': 'Round',
        'Time(s)': 'Time(s)',
        'In Tokens': 'In Tokens',
        'Out Tokens': 'Out Tokens',
        'Total Tokens': 'Total Tokens',
        'sending_task': 'Sending task to {}',
        'error_loading_config': "Error loading configuration: {}",
        'config_file_error': "Please check the configuration file path and format.",
        'config_not_loaded': "Configuration not loaded.",
        'llm_config_not_found': "Missing 'llm' configuration.",
        'trustoken_register_instruction': (
            "The current environment lacks a configuration file. Please register for a Trustoken account to use the free API token.\n"
            "Open https://api.trustoken.ai/register to register.\n"
            "After registration, log in and visit the 'Token' page at the top, or navigate to: https://api.trustoken.ai/token\n"
            "Click the 'copy' button to copy your token to the clipboard and paste it here.\n"
            "Alternatively, you can exit now, and manually edit the configuration file {} to configure your existing LLM token."
        ),
        'prompt_token_input': "Please paste the token and press Enter (type 'exit' to quit): ",
        'exit_token_prompt': "Exiting token input process.",
        'no_token_detected': "No token detected.",
        'invalid_token': "The entered token is invalid. Ensure it starts with 'sk-' followed by the correct characters, or type 'exit' to quit.",
        'token_saved': "Token saved to {}",
        'token_save_error': "Error saving token: {}"
    }
}

lang = 'en'
language, _ = locale.getlocale()
if language:
    language = language.lower()
    if language.find('china') >=0 or language.find('chinese') >= 0 or language.find('zh_') >= 0:
        lang = 'zh'

def T(key, *args):
    msg = MESSAGES[lang][key]
    return msg.format(*args) if args else msg



import sys
import os
import json
from pathlib import Path

import requests
from bs4 import BeautifulSoup

API_URL = "https://ai.xxyy.eu.org/api/publish"
CERT_PATH = Path('client.crt')

def upload_article(api_url, html_file_path):
    if not os.path.isfile(html_file_path):
        print(f"错误: 文件 {html_file_path} 不存在")
        return False

    with open(html_file_path, 'r', encoding='utf-8') as file:
        html_content = file.read()

    soup = BeautifulSoup(html_content, 'html.parser')
    title = soup.find('span', class_='r4').get_text()

    try:
        files = {
            'content': (html_file_path, html_content, 'text/html')
        }
        meta = {'author': os.getlogin()}
        data = {
            'title': title,
            'metadata': json.dumps(meta)
        }        
        
        response = requests.post(
            api_url,
            files=files,
            data=data,
            cert=str(CERT_PATH),
            verify=True
        )
        
        if response.status_code == 201:
            print("文章上传成功!")
            print("访问URL:", response.json())
            return True
        else:
            print(f"上传失败 (状态码: {response.status_code}):", response.text)
            return False
    except Exception as e:
        print("发生错误:", str(e))
        return False

if __name__ == "__main__":
    if len(sys.argv) != 2:
        print("用法: python publish.py 'path/to/article.html'")
        sys.exit(1)
    
    html_file = sys.argv[1]
    
    print(f"正在上传文件: {html_file})")
    upload_article(API_URL, html_file)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

from abc import ABC, abstractmethod

class Runtime(ABC):
    @abstractmethod
    def install_packages(self, packages):
        pass

    @abstractmethod
    def getenv(self, name, desc=None):
        pass

class ConsoleInterface(ABC):
    @abstractmethod
    def print(self, *args, sep=' ', end='\n', file=None, flush=False):
        pass

    @abstractmethod
    def input(self, prompt=''):
        pass

    @abstractmethod
    def status(self, msg):
        pass